package jitas.admin;
import java.awt.event.ActionEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.JOptionPane;
import jitas.exception.UserNotFoundException;

/**
 *This class contains the Graphical user interface code
 *for JITAS Admin Tool
 *@author DeathMarch(c314g2)
 *@version 08 October 2007
 */
public class AdminFrame extends javax.swing.JFrame {
    
	private static final String about = "JITAS Admin 1.0\nCreated 2007 by:\n" +
										"  David Thomson\n" +
										"  Farah Mohamad\n" +
										"  Louis Sayers\n" +
										"  Lu Pan\n" +
										"  Robert Lechte\n" +
										"  Yuganeethen Yugaraja\n"; 
										
	
	
	GUIHelper helper;
    /**
	 * 
	 */
	private static final long serialVersionUID = 9092596771875227977L;
	/** Creates new form AdminFrame */
    public AdminFrame() {
    	try {
    	helper = new GUIHelper();
    	
    	
    	
    	} catch (IOException ex) {
    		String error = "Setup Failed! IOException occured. Likely cause is a problem"+
    						" with loading xml documents in JITAS";
    		JOptionPane.showMessageDialog(this, error , "Data Loading Error", JOptionPane.ERROR_MESSAGE);
    	}
    	
    	try {
    	
        initComponents();
    	}
    	catch (Exception ex) {
    		String error = "A critical error occurred within JITAS. The error log MAY contain useful" +
    				"information regarding JITAS failure.";
    		JOptionPane.showMessageDialog(this, error + "\n" + ex, "JITAS System Error" , JOptionPane.ERROR_MESSAGE);
    		
    	}
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     * @throws FileNotFoundException 
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">                          
    private void initComponents() throws FileNotFoundException {
        jScrollPane1 = new javax.swing.JScrollPane();
        studentList = new javax.swing.JList();
        studentLabel = new javax.swing.JLabel();
        detailsLabel = new javax.swing.JLabel();
        usernameLabel = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        domainTable = new javax.swing.JTable();
        filterComboBox = new javax.swing.JComboBox();
        filterLabel = new javax.swing.JLabel();
        removeButton = new javax.swing.JButton();
        addButton = new javax.swing.JButton();
        mainMenuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();
        studentMenu = new javax.swing.JMenu();
        createStudentMenuItem = new javax.swing.JMenuItem();
        deleteStudentMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();
        
        domainTable.setDefaultRenderer(String.class, new ColourCellRenderer());

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Jitas Admin");
        studentList.setModel(new javax.swing.AbstractListModel() {
            /**
			 * 
			 */
			private static final long serialVersionUID = 5996242878103481178L;
			String[] strings = helper.getStudentUsernameStrings();
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        studentList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        studentList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                studentListValueChanged(evt);
            }
        });
        
        domainTable.getSelectionModel().addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                domainTableSelectionValueChanged(evt);
            }
        });
        
      
        
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }

			
        });
        
        createStudentMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createStudentMenuItemActionPerformed(evt);
            }
        });
        
        deleteStudentMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteStudentMenuItemActionPerformed(evt);
            }
        });
        
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        
        removeButton.setText("Remove");
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        });

        addButton.setText("Add");
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });
        
        addButton.setEnabled(false);
		removeButton.setEnabled(false);
        
        
        domainTable.setModel(helper.getStudentTableModel());
        usernameLabel.setText("Username:");
        
        
        if(studentList.getModel().getSize() > 0) {
        	studentList.setSelectedIndex(0);
        }
        	
        
        jScrollPane1.setViewportView(studentList);

        studentLabel.setText("Students:");

        detailsLabel.setText("Details");

       
        jScrollPane2.setViewportView(domainTable);

        filterComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "All Domains" }));
        filterComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                filterComboBoxActionPerformed(evt);
            }
        });

        for (String domain: helper.getAllDomainNames()) {
        	filterComboBox.addItem(domain);
        }
        
        
        
        
        filterLabel.setText("Filter:");

        

        
        

        fileMenu.setText("File");
        exitMenuItem.setText("Exit");
        fileMenu.add(exitMenuItem);

        mainMenuBar.add(fileMenu);

        studentMenu.setText("Student");
        createStudentMenuItem.setText("Create New Student...");
        studentMenu.add(createStudentMenuItem);

        deleteStudentMenuItem.setText("Delete Student");
        studentMenu.add(deleteStudentMenuItem);

        mainMenuBar.add(studentMenu);

        helpMenu.setText("Help");
        aboutMenuItem.setText("About...");
        helpMenu.add(aboutMenuItem);

        mainMenuBar.add(helpMenu);

        setJMenuBar(mainMenuBar);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 146, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(layout.createSequentialGroup()
                            .add(filterLabel)
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                            .add(filterComboBox, 0, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .add(studentLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, detailsLabel)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, usernameLabel)
                    .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 330, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(addButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(removeButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(33, 33, 33)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(studentLabel)
                            .add(detailsLabel)))
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(filterLabel)
                        .add(filterComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 260, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createSequentialGroup()
                        .add(usernameLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 202, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 12, Short.MAX_VALUE)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(removeButton)
                            .add(addButton))))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>                        

    private void studentListValueChanged(javax.swing.event.ListSelectionEvent evt) {  
    	
    	String username = (String) studentList.getSelectedValue();
    	if(studentList.getModel().getSize() == 0) {
        	usernameLabel.setText("Username: ");
    	}
    	else {
    		usernameLabel.setText("Username: " + username);
    	}
    	helper.setCurrentUser(username);
    	domainTable.setModel(helper.getStudentTableModel());
    }          
    
    
    
    private void domainTableSelectionValueChanged(javax.swing.event.ListSelectionEvent evt) { 
    	try {
    	int selectedRow = domainTable.getSelectedRow();
    	if(selectedRow >= 0) {
	    	boolean allowedInDomain = helper.isAllowedInDomain(selectedRow);
	    	if(allowedInDomain) {
	    		addButton.setEnabled(false);
	    		removeButton.setEnabled(true);
	    	} else {
	    		addButton.setEnabled(true);
	    		removeButton.setEnabled(false);
	    	}
    	} else {
    		addButton.setEnabled(false);
    		removeButton.setEnabled(false);
    	}
    	} catch (UserNotFoundException ex) {
    		String errorMessage = "Unexpectedly the student could not be found";
    		JOptionPane.showMessageDialog(this, errorMessage , "Student Error", JOptionPane.ERROR_MESSAGE);
    	}
    }          
    
    private void exitMenuItemActionPerformed(ActionEvent evt) {
		System.exit(0);
		
	}
    
    private void createStudentMenuItemActionPerformed(ActionEvent evt) {
		new NewStudentFrame(this, helper).setVisible(true);
		
	}
    
    private void deleteStudentMenuItemActionPerformed(ActionEvent evt) {
    	String username = (String) studentList.getSelectedValue();
    	int choice = JOptionPane.showConfirmDialog(
    		    this,
    		    "Deleting a student will remove the student\n" +
    		    "from all domains, and will also delete their\n" +
    		    "username, meaning they will be totally\n" +
    		    "removed from the system. Are you sure you\n" +
    		    "wish to delete \"" +username +"\"?",
    		    "WARNING: Delete \"" + username + "\"?",
    		    JOptionPane.YES_NO_OPTION);
		
		
		if (choice == 0) {
			
			try {
				
				helper.deleteUser(username);
				
				try {
					studentList.setModel(new javax.swing.AbstractListModel() {
						private static final long serialVersionUID = -8985936389726414274L;
						String[] strings = helper.getStudentUsernameStrings();
					    public int getSize() { return strings.length; }
					    public Object getElementAt(int i) { return strings[i]; }
					});
					if(studentList.getModel().getSize() > 0) {
			        	studentList.setSelectedIndex(0);
			        }
					
				} catch (FileNotFoundException e) {
					String error = "A critical error occurred within JITAS. The error log MAY contain useful" +
					" information regarding JITAS failure.";
					JOptionPane.showMessageDialog(this, error , "JITAS System Error", JOptionPane.ERROR_MESSAGE);
				}
				
				JOptionPane.showMessageDialog(this, "User \"" + username + "\" deleted successfully", "User Deleted", JOptionPane.INFORMATION_MESSAGE);
				
			} catch (Exception e) {
				String error = "Something has gone wrong while deleting user: \"" + username + "\"" + e;
				JOptionPane.showMessageDialog(this, error , "Student Data Error", JOptionPane.ERROR_MESSAGE);
			}
			
		}
	}
    
    private void aboutMenuItemActionPerformed(ActionEvent evt) {
    	 JOptionPane.showMessageDialog(this, about, "About JITAS Admin 1.0", JOptionPane.INFORMATION_MESSAGE);
	}
    
    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {
    	try {
    		
    		int choice = JOptionPane.showConfirmDialog(
        		    this,
        		    "This will totally delete all the student model information for this domain.\n" +
        		    "Are you sure you want to do this?",
        		    "WARNING: Delete Student Model?",
        		    JOptionPane.YES_NO_OPTION);
    		
    		if (choice == 0) {
    			int selectedRow = domainTable.getSelectedRow();
    			helper.removeDomain(selectedRow);
    			domainTable.setModel(helper.getStudentTableModel());
    		}
		} catch (Exception e) {
			String error = "A critical error occurred within JITAS. The error log MAY contain useful" +
			" information regarding JITAS failure.";
			JOptionPane.showMessageDialog(this, error , "JITAS System Error", JOptionPane.ERROR_MESSAGE);
		}
    }

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {
    	try {
    		int selectedRow = domainTable.getSelectedRow();
			helper.addDomain(selectedRow);
			domainTable.setModel(helper.getStudentTableModel());
    	} catch (Exception e) {
			String error = "A critical error occurred within JITAS. The error log MAY contain useful" +
			" information regarding JITAS failure.";
			JOptionPane.showMessageDialog(this, error+"\n" + e , "JITAS System Error", JOptionPane.ERROR_MESSAGE);
		}
    }
    
    private void filterComboBoxActionPerformed(java.awt.event.ActionEvent evt) {
    	String selectedItem = (String)filterComboBox.getSelectedItem();
    	if(selectedItem.equals("All Domains")){
    		helper.setFilter(null);
    	} else { 
    		helper.setFilter(selectedItem);
    	}
    	
    	try {
			studentList.setModel(new javax.swing.AbstractListModel() {
				private static final long serialVersionUID = -5389031433772850778L;
				String[] strings = helper.getStudentUsernameStrings();
			    public int getSize() { return strings.length; }
			    public Object getElementAt(int i) { return strings[i]; }
			});
			if(studentList.getModel().getSize() > 0) {
	        	studentList.setSelectedIndex(0);
	        }
			
		} catch (FileNotFoundException e) {
			String error = "A critical error occurred within JITAS. The error log MAY contain useful" +
			"information regarding JITAS failure.";
			JOptionPane.showMessageDialog(this, error , "JITAS System Error", JOptionPane.ERROR_MESSAGE);
		}
    	
    }
    
    
    public void updateAfterNewStudent(String newuser) {
    	
    	String selectedItem = (String)filterComboBox.getSelectedItem();
    	if(selectedItem.equals("All Domains")){
    		helper.setFilter(null);
    	} else { 
    		helper.setFilter(selectedItem);
    	}
    	
    	try {
			studentList.setModel(new javax.swing.AbstractListModel() {
				private static final long serialVersionUID = -9023928699746321280L;
				String[] strings = helper.getStudentUsernameStrings();
			    public int getSize() { return strings.length; }
			    public Object getElementAt(int i) { return strings[i]; }
			});
			if(studentList.getModel().getSize() > 0) {
	        	studentList.setSelectedIndex(0);
	        }
			
		} catch (FileNotFoundException e) {
			String error = "A critical error occurred within JITAS. The error log MAY contain useful" +
			"information regarding JITAS failure.";
			JOptionPane.showMessageDialog(this, error , "JITAS System Error", JOptionPane.ERROR_MESSAGE);
		}
		
		String currentuser = null;
		
		for (int i = 0; i <studentList.getModel().getSize(); i++) {
			currentuser = (String) studentList.getModel().getElementAt(i);
			
			if (currentuser.equals(newuser)) {
				studentList.setSelectedIndex(i);
				return;
			}
		}
	    	
		filterComboBox.setSelectedIndex(0);
		
		for (int i = 0; i <studentList.getModel().getSize(); i++) {
			currentuser = (String) studentList.getModel().getElementAt(i);
			if (currentuser.equals(newuser)) {
				studentList.setSelectedIndex(i);
				return;
			}
		}
    }
    
    public void reenableControls() {
    	studentList.setEnabled(true);
    	domainTable.setEnabled(true);
    	filterComboBox.setEnabled(true);
    }
    
    
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new AdminFrame().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify                     
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JButton addButton;
    private javax.swing.JMenuItem createStudentMenuItem;
    private javax.swing.JMenuItem deleteStudentMenuItem;
    private javax.swing.JLabel detailsLabel;
    private javax.swing.JTable domainTable;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JComboBox filterComboBox;
    private javax.swing.JLabel filterLabel;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JMenuBar mainMenuBar;
    private javax.swing.JButton removeButton;
    private javax.swing.JLabel studentLabel;
    private javax.swing.JList studentList;
    private javax.swing.JMenu studentMenu;
    private javax.swing.JLabel usernameLabel;
    // End of variables declaration                   
    
}
