package jitas.core;

import java.util.SortedMap;

/**
 * The interface class for the Java Intelligent Tutor Authoring Shell (JITAS)
 * system selection algorithm. Interfacing client will use this class to implement
 * domain-specific system-chosen algorithm.
 * @author	DeathMarch (c314g2)
 * @version	12 August 2007
 */
public interface ProblemSelectionAlgorithm {
	/**
	 * Return a Problem object that has been calculated by a specific algorithm
	 * This design pattern aims to increase flexibility for client to add/use their
	 * own algorithm for system chosen problems 
	 * @param student The student
	 * @param domain The Domain
	 * @param problems The problems to choose from
	 * @param subdomainName The name of the subdomain
	 * @return Problem The next problem computed by the algorithm
	 */
	public Problem getNextProblem(Student student, Domain domain, SortedMap<Integer, Problem> problems, String subdomainName);

}
