package jitas.core;

import java.io.Serializable;

/**
 * A solution to a problem
 * 
 * @author DeathMarch (c314g2)
 * @version 08 October 2007
 */
public class Solution implements Serializable {
	private static final long serialVersionUID = 4451651798243441293L;
	private String solutionContent;
	
	/**
	 * Default constructor that makes a new empty content
	 */
	public Solution() {
		solutionContent = "";
	}
	
	/**
	 * Constructor that takes in a String as the content
	 * eg: (("CASE" "N")("GENDER" "N")("ARTICLE" "D")("ANSWER" "graue"))
	 * This is used by the xml parser
	 * @param content The String of content
	 */
	public Solution(String content) {
		solutionContent = content;
	}
	
	/**
	 * Get the solution content as a String of key-value pairs, 
	 * with the first and last bracket removed.
	 * eg a solution of: 
	 * 	(("CASE" "N")("GENDER" "N")("ARTICLE" "D")("ANSWER" "graue"))
	 * will return 
	 * 	("CASE" "N")("GENDER" "N")("ARTICLE" "D")("ANSWER" "graue")
	 * 
	 * This is so the solution can be asserted.
	 * @return The solution content
	 */
	public String getContent() {
		return solutionContent.substring(1, solutionContent.length()-1);
	}

	/**
	 * Set the solution of a Problem
	 * @param content The solution content
	 */
	public void setSolution(String content){
		solutionContent = content;
	}	
}
