package jitas.xml.parser;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import jitas.core.Domain;
import jitas.core.Subdomain;
import jitas.exception.DomainNotFoundException;
import jitas.util.Settings;

public class DomainParser extends XMLParser {

    private Document parsedDoc = null;
    private Settings settings;
    private String domainName = "";
    
    /**
     * Constructs a parser and loads a document
     * @param domainName
     * @throws DomainNotFoundException
     */
    public DomainParser(String domainName) throws DomainNotFoundException {
    	//System.out.println("PARSING NOW");
    	settings = Settings.getInstance();
    	this.domainName = domainName;
        try {
        	
        	parsedDoc = super.loadXML(settings.getRootPath() + domainName + "/" + domainName + ".xml");
        } catch (SAXException se) {
        	System.err.println(se.getMessage());
        } catch (IOException ioe) {
        	System.err.println("FILE NOT FOUND");
        	throw new DomainNotFoundException("DomainParser.Constructor: Domains file not found at: " + domainName);
        }
    }
    
    /**
     * Gets a Domain based on the document supplied in the constructor
     * @return domain
     * @throws DomainNotFoundException
     */
    public Domain processTree() throws DomainNotFoundException {
    	/*Declear useful variables*/
    	String name="", longName="", description="", type="", subdomainTerm="";
    	boolean userCreationAllowed = false;  	
    	Set<Subdomain> subdomains = new HashSet<Subdomain>();
    	
    	if(super.isFileEmpty(settings.getRootPath() + domainName + "/" + domainName + ".xml")){
			/* Create a student model*/
	        Element domain = parsedDoc.createElement("domain");
	        /*Append the student model element to the root element*/
			parsedDoc.appendChild(domain);
			super.writeToXML(parsedDoc, settings.getRootPath() + domainName + "/" + domainName + ".xml");
			return null;
		}
    	
    	/* Get the root element of the document*/
    	NodeList domainTag = parsedDoc.getElementsByTagName("domain");
    	Node domainNode = domainTag.item(0);
    	
    	/*Iterate through each domain to get the wanted components*/
    	for (Node child = domainNode.getFirstChild(); child!=null; child = child.getNextSibling()) {	
    		if(child.getNodeName().equals("name")) {    			
    			name = child.getTextContent();
    		} else if (child.getNodeName().equals("long-name")) {
    			longName = child.getTextContent();    			
    		} else if (child.getNodeName().equals("description")) {
    			description = child.getTextContent();
    		} else if (child.getNodeName().equals("user-creation-allowed")) {
    			userCreationAllowed = child.getTextContent().equals("yes");
    		} else if (child.getNodeName().equals("type")) {
    			type = child.getTextContent();
    		} else if (child.getNodeName().equals("subdomain-term")) {
    			subdomainTerm = child.getTextContent();
    		} else if (child.getNodeName().equals("subdomains")) {
    			
    			/*Get a list of subdomain elements*/
    			NodeList subdomainTags = child.getOwnerDocument().getElementsByTagName("subdomain");
    			
    			/*Iterate through each subdomain*/
    			for (int i = 0; i < subdomainTags.getLength(); i++) {
    				String nameOfSubdomain="", descriptionOfSubdomain="";    				
    				Node subdomain = subdomainTags.item(i);
    				
    				/*Retrieve the components of each subdomain*/
    				for (Node subdomainChild = subdomain.getFirstChild(); subdomainChild!=null; subdomainChild = subdomainChild.getNextSibling()) {
    					if(subdomainChild.getNodeName().equals("name")) {
    		    			nameOfSubdomain = subdomainChild.getTextContent();
    		    		} else if (subdomainChild.getNodeName().equals("description")) {
    		    			descriptionOfSubdomain = subdomainChild.getTextContent();
    		    		} 
    				}
    				
    				/*Construct a subdomain and add it to the subdomain collection*/
    				Subdomain sd = new Subdomain(nameOfSubdomain, descriptionOfSubdomain, name);
    				subdomains.add(sd);
    			}
    		} 
    	}
    	
    	/*Construct a domain with its components and return it back*/
    	Domain domain = new Domain(name, longName, description,userCreationAllowed,type, subdomainTerm, subdomains);
    	//domain.setStudentModelManager(new StudentModelManager(domain));
    	return domain;
    }
}
