package jitas.xml.parser;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import jitas.util.Logger;

/**
 * The abstract class that provides the basic functions for users to parse
 * the content of XML files.
 * 
 * @author	DeathMarch (c314g2)
 * @version	04 July 2007
 */
public abstract class XMLParser {
	
	/**
	 * Load the XML file with given path name.
	 * @param pathName
	 * @return parsedDoc
	 */
	public Document loadXML(String pathName) throws IOException, SAXException {
		DocumentBuilder db = null;
		Document parsedDoc = null; 
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

        /*Create the document builder*/
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            System.err.println(pce);
        }
        db.setErrorHandler(new DefaultHandler());
        
        /*If the file is empty, return an empty new Document */
        if(isFileEmpty(pathName)) return db.newDocument();
        
        /*Load the xml doc*/
        try {
        	parsedDoc = db.parse(pathName);
        } catch (IOException e) {
        	//System.err.println("Loading " + pathName + "...Failed!");
        	throw e;
        }
        
        return parsedDoc;
	}
	
	/**
	 * Using DOM parser to parse the XML file and return the collections of 
	 * needed object
	 */
	public abstract Object processTree() throws Exception;
	
	/*
	 * If the parsedDoc has no child nodes, it is assumed that
	 * the document is empty.  This method will return true if the
	 * document has no children, otherwise returns false
	 */
	public boolean documentIsEmpty(Document parsedDoc) {
		System.out.println(parsedDoc.getFirstChild());
		if(!parsedDoc.hasChildNodes()){	
			return true;
		}
		return false;
	}
	/**
	 * Write the content to XML file with given document and path name
	 * If something goes wrong, and the file can't write, we don't write anything.
	 * @param parsedDoc
	 */
	public void writeToXML(Document parsedDoc, String pathName){
		try{
			if(!checkWeCanWriteToFile(pathName))return;
			OutputFormat format = new OutputFormat(parsedDoc);
			format.setIndenting(true);
			
			/* To generate a file output use fileoutputstream instead of system.out */
			XMLSerializer serializer = new XMLSerializer(
			new FileOutputStream(new File(pathName)), format);
			serializer.serialize(parsedDoc);
		} catch(IOException ie) {
			ie.printStackTrace();
		}	
	}
	
	/*
	 * Returns true if the file exists, if the file didn't exist, the method
	 * will attempt to create it, and if that fails, then this method will return 
	 * false.
	 */
	public boolean checkWeCanWriteToFile(String pathName) {
		 /*See if the file actually exists! if not, create it.*/
        File file = new File(pathName);
        if(!file.exists()) {
        	Logger.addEntry("XMLWriter had to create " + pathName + " as this did not exist!\n");
        	try {
				file.createNewFile();
			} catch (IOException e) {
				Logger.addEntry("IOException was caught in XMLWriter!!! aborting file creation.\n\n " +
						"printStackTrace: \n" + e.toString() +"\n\n\n");
				return false;
			}
        }
        return true;
	}
	
	/*
	 * tests to see whether a file is empty or not
	 * param pathName the location of the file
	 * return true if the file is empty, else false.
	 */
	public boolean isFileEmpty(String pathName) {
		File f = new File(pathName);
    	try {
			FileReader reader = new FileReader(f);
			if(reader.read() == -1) return true;
		} catch (FileNotFoundException e) {
		} catch (IOException e) {}
		
		return false;
	}
}
