package jitas.xml.writer;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import jitas.core.Domain;
import jitas.util.Settings;

/**
 * A class that provides functions for users to write to domains.xml.
 * 
 * @author	DeathMarch (c314g2)
 * @version	15 May 2007
 */
public class AllDomainsWriter extends XMLWriter {
	
	private Domain d = null;
	private Document parsedDoc = null;
	private String pathName = "";
	
	/**
	 * Constructor. Pass the parameters to the superclass to load
	 * the XML file.
	 * @param d the domain needs to be added
	 * @param pathName the path to the file that contains all the domain names
	 */
	public AllDomainsWriter(Domain d, String pathName){
		this.d = d;
		this.pathName = pathName;
		parsedDoc = super.loadXML(pathName);
	}
	
	/**
	 * Constructor. Pass the parameters to the superclass to load
	 * the XML file.
	 * @param d the domain needs to be added
	 */
	public AllDomainsWriter(Domain d){
		this.d = d;
		parsedDoc = super.loadXML(Settings.getInstance().getFullPathToAllDomains());
	}

	
	/**
	 * Add a domain to the domains XML file.
	 */
	public void addDomain(){
		/* Create a domain element*/
        Element domain = parsedDoc.createElement("domain");
        
        /* Create name element and attach it to domain */
		Element name = parsedDoc.createElement("name");
		Text nameText = parsedDoc.createTextNode(d.getName());
		name.appendChild(nameText);
		domain.appendChild(name);
		
		parsedDoc.getFirstChild().appendChild(domain);
		
		/*Write to the XML file*/
		super.writeToXML(parsedDoc,pathName);
	}
}
