package jitas.xml.writer;

import jitas.core.Student;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * A class that provides functions for users to write to students.xml.
 * 
 * @author	DeathMarch (c314g2)
 * @version	05 August 2007
 */
public class StudentWriter extends XMLWriter{
	
	private Student student= null;
	private Document parsedDoc = null;
	private String pathName = "";
	
	/**
	 * Constructor. Pass the parameters to the superclass to load
	 * the XML file. If you only want to change the password, 
	 * then pass in an empty domainName.
	 */
	public StudentWriter(Student student, String pathName){
		this.student = student;
		this.pathName = pathName;
		parsedDoc = super.loadXML(pathName);
	}
	
	/**
	 * Add a student to the student XML file.
	 */
	public void addStudent(){
		/* Create a student model*/
        Element studentEle = parsedDoc.createElement("student");
        
        /* Create username element and attach it to studentModel */
		Element usernameEle = parsedDoc.createElement("username");
		Text usernameText = parsedDoc.createTextNode(student.getUserName());
		usernameEle.appendChild(usernameText);
		
		/* Create password element and attach it to studentModel */
		Element passwordEle = parsedDoc.createElement("password");
		
		//System.err.println("password: " + student.getPassword());
		//System.err.println("password: " + Encryptor.getEncryptor().encrypt(student.getPassword()));
		
		
		Text passwordText = parsedDoc.createTextNode(student.getPassword());
		passwordEle.appendChild(passwordText);
		
		/* Create an allowed-domains element  */
		Element allowedDomainsEle = parsedDoc.createElement("allowed-domains");
		
		for(String domainName : student.getAllowedDomains()) {
			Element domainEle = parsedDoc.createElement("domain");
			Text domainText = parsedDoc.createTextNode(domainName);
			domainEle.appendChild(domainText);
			/*Append the domain to the allowed-domains element*/
			allowedDomainsEle.appendChild(domainEle);
		}
		
		/*Append the created username and password elements to the student model element*/
		studentEle.appendChild(usernameEle);
		studentEle.appendChild(passwordEle);
		studentEle.appendChild(allowedDomainsEle);
		
		/*Append the student model element to the root element*/
		parsedDoc.getFirstChild().appendChild(studentEle);
		
		/*Write to the XML file*/
		super.writeToXML(parsedDoc,pathName);
	}
	
	/**
	 * Removes a student to the student XML file.
	 */
	public void removeStudent(){
		NodeList usernames = parsedDoc.getElementsByTagName("username");
		String userName = student.getUserName();
		
		for (int i = 0; i < usernames.getLength() ; i++) {
			Node n = usernames.item(i);	
			
			if (n.getTextContent().equals(userName)) {
				Node studentModelNode = n.getParentNode();
				
				Node parentNode = studentModelNode.getParentNode();
				
				parentNode.removeChild(studentModelNode);
			}
		}
		
		/*Write to the XML file*/
		super.writeToXML(parsedDoc,pathName);
	}
	
	/**
	 * Save the new password that student changed to xml file
	 */
	public void changePassword(){
		NodeList list = parsedDoc.getElementsByTagName("student");
		
		/*Go through each student model to find the one needs to be modified*/
		for(int i = 0; i < list.getLength(); i++) {
			Node n = list.item(i);
			boolean flag = false;
			String userName = student.getUserName();
			
			/*Find the correct user and save the new password to the xml file*/
			for(Node child = n.getFirstChild(); child != null; child = child.getNextSibling()){     	   
		    	   if(child.getNodeName().equals("username")) {
		    		   if(child.getTextContent().equals(userName)){
		    			   flag = true;
		    		   } 
		    	   } else if(flag==true && child.getNodeName().equals("password")){
		    			   child.setTextContent(student.getPassword());
		    			   break;
		    	   } 
			}
			
			/*If it is done, jump out the loop*/
			if(flag == true){
				break;
			}
		}
		
		/*Write to the XML file*/
		super.writeToXML(parsedDoc,pathName);
	}
	
	/**
	 * Adds a domain to the set of Allowed domains which a student may log into.
	 * We are given the domain name in the constructor.
	 */
	public void addAllowedDomain(String domainName){
		NodeList list = parsedDoc.getElementsByTagName("student");
		
		/*Go through each student model to find the one needs to be modified*/
		for(int i = 0; i < list.getLength(); i++) {
			Node n = list.item(i);
			boolean flag = false;
			String userName = student.getUserName();
			
			/*Find the correct user and save the new domain to the xml file*/
			for(Node child = n.getFirstChild(); child != null; child = child.getNextSibling()){     	   
		    	   if(child.getNodeName().equals("username")) {
		    		   if(child.getTextContent().equals(userName)){
		    			   flag = true;
		    		   } 
		    	   } else if(flag==true && child.getNodeName().equals("allowed-domains")){
		    		   //NodeList domainChildren = child.getChildNodes();
		    		   
		    		 /* check to make sure the domain doesn't already exist */
		    		   /*for(int counter=0; counter < domainChildren.getLength(); counter++) {
		    			   if(domainChildren.item(counter).getTextContent().equals(domainName)) 
		    				   return;
		    		   }*/
		    		   /* create the domain Element */
		    		   Element domainEle = parsedDoc.createElement("domain");
		    			Text domainText = parsedDoc.createTextNode(domainName);
		    			domainEle.appendChild(domainText);
		    			
		    			/*Add the domain element to the allowed-domains element*/
		    			child.appendChild(domainEle);
		    	   } 
			}
			
			/*If it is done, jump out the loop*/
			if(flag == true){
				break;
			}
		}
		
		/*Write to the XML file*/
		super.writeToXML(parsedDoc,pathName);
	}
	
	/**
     * Removes a single domain name from the allowed domains set
     * @param domainName
     */
	public void removeDomain(String domainName) {
		NodeList usernames = parsedDoc.getElementsByTagName("username");
		String userName = student.getUserName();
		
		for (int i = 0; i < usernames.getLength() ; i++) {
			Node n = usernames.item(i);	
			
			if (n.getTextContent().equals(userName)) {
				
				for(Node sibling = n.getNextSibling(); sibling != null; sibling = sibling.getNextSibling()){
			    	  if(sibling.getNodeName().equals("allowed-domains")){
			    		   NodeList domainChildren = sibling.getChildNodes();
			    		   
			    		 /* check to make sure the domain doesn't already exist */
			    		   for(int counter=0; counter < domainChildren.getLength(); counter++) {
			    			   if(domainChildren.item(counter).getTextContent().equals(domainName)) {
			    				   sibling.removeChild(domainChildren.item(counter));
			    				   break;
			    			   }
			    		   }
			    		   break;
			    	   } 
			      
				}
				break;
			}
		}
		/*Write to the XML file*/
		super.writeToXML(parsedDoc,pathName);
	}
	
	
}
