package jitas.unittest;

import java.util.SortedMap;

import jitas.core.*;
import jitas.exception.*;
import junit.framework.TestCase;

/* 
 * @author	DeathMarch (c314g2)
 * @version	8 October 2007
 */
public class AcceptanceTest extends TestCase {
	
	TutorInterface tutor;

	protected void setUp() throws Exception {
		super.setUp();
		
		Process p = Runtime.getRuntime().exec("./testfiles/copy-out-testdomains.sh");
		p.waitFor();
		
		tutor = new Tutor("testfiles/testdomains/");
		
		tutor.loadDomain("german");
		
		tutor.addUser("SherlockHolmes", "detective", "german");
		tutor.login("SherlockHolmes", "detective", "german");
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		
		tutor = null;
		
		Process p = Runtime.getRuntime().exec("./testfiles/delete-testdomains.sh");
		p.waitFor();
	}

	public final void testTutor() {
		
		// Testing successful Tutor() constructor
		try {
			tutor = new Tutor();
			assertNotNull(tutor);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public final void testTutorString() {
		
		// Testing successful Tutor(String domainsFilePath) constructor
		assertNotNull(tutor);
	}

	public final void testLogin() {
		
		// Setup
		try {
			tutor.addUser("JohnWatson", "biographer", "german");
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing successful login
		try {
			assertTrue(tutor.getLoginStatus("SherlockHolmes", "german"));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing successful login again
		try {
			assertTrue(tutor.getLoginStatus("SherlockHolmes", "german"));
			tutor.login("SherlockHolmes", "detective", "german");
			assertTrue(tutor.getLoginStatus("SherlockHolmes", "german"));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotFoundException
		try {
			assertFalse(tutor.getLoginStatus("JohnWatson", "german"));
			tutor.login("JohnWatson", "biographer", "british");
			
			System.err.println("Expected DomainNotFoundException to be thrown!");
			fail();
		} catch (DomainNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotLoadedException
		try {
			assertFalse(tutor.getLoginStatus("JohnWatson", "german"));
			tutor.login("JohnWatson", "biographer", "kermit");
			
			System.err.println("Expected DomainNotLoadedException to be thrown!");
			fail();
		} catch (DomainNotLoadedException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing InvalidLoginException
		try {
			assertFalse(tutor.getLoginStatus("JohnWatson", "german"));
			tutor.login("JohnWatson", "friend", "german");
			
			System.err.println("Expected InvalidLoginException to be thrown!");
			fail();
		} catch (InvalidLoginException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing UserNotFoundException
		try {
			tutor.login("ConanDoyle", "author", "german");
			
			System.err.println("Expected UserNotFoundException to be thrown!");
			fail();
		} catch (UserNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public final void testLogout() {
		
		// Setup
		try {
			tutor.addUser("JohnWatson", "biographer", "german");
			tutor.login("JohnWatson", "biographer", "german");
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing successful logout
		try {
			assertTrue(tutor.getLoginStatus("SherlockHolmes", "german"));
			tutor.logout("SherlockHolmes", "german");
			assertFalse(tutor.getLoginStatus("SherlockHolmes", "german"));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotFoundException
		try {
			assertTrue(tutor.getLoginStatus("JohnWatson", "german"));
			tutor.logout("JohnWatson", "british");
			
			System.err.println("Expected DomainNotFoundException to be thrown!");
			fail();
		} catch (DomainNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotLoadedException
		try {
			assertTrue(tutor.getLoginStatus("JohnWatson", "german"));
			tutor.logout("JohnWatson", "kermit");
			
			System.err.println("Expected DomainNotLoadedException to be thrown!");
			fail();
		} catch (DomainNotLoadedException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing UserNotFoundException
		try {
			tutor.logout("ConanDoyle", "german");
			
			System.err.println("Expected UserNotFoundException to be thrown!");
			fail();
		} catch (UserNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing UserNotLoggedInException
		try {
			assertFalse(tutor.getLoginStatus("SherlockHolmes", "german"));
			tutor.logout("SherlockHolmes", "german");
			
			System.err.println("Expected UserNotLoggedInException to be thrown!");
			fail();
		} catch (UserNotLoggedInException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public final void testGetLoginStatus() {
		
		// Setup
		try {
			tutor.addUser("JohnWatson", "biographer", "german");
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing successful getLoginStatus
		try {
			assertTrue(tutor.getLoginStatus("SherlockHolmes", "german"));
			assertFalse(tutor.getLoginStatus("JohnWatson", "german"));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing UserNotFoundException
		try {
			tutor.getLoginStatus("ConanDoyle", "german");
			
			System.err.println("Expected UserNotFoundException to be thrown!");
			fail();
		} catch (UserNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public final void testAddUser() {
		
		// Setup
		try {
			tutor.loadDomain("kermit");
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing successful addUser
		try {
			tutor.addUser("ConanDoyle", "author", "german");
			tutor.login("ConanDoyle", "author", "german");
			assertTrue(tutor.getLoginStatus("ConanDoyle", "german"));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
			
		// Testing AddUserDeniedException
		try {
			tutor.addUser("JohnWatson", "biographer", "kermit");
			
			System.err.println("Expected AddUserDeniedException to be thrown!");
			fail();
		} catch (AddUserDeniedException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotFoundException
		try {
			tutor.addUser("JohnWatson", "biographer", "british");
			
			System.err.println("Expected DomainNotFoundException to be thrown!");
			fail();
		} catch (DomainNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotLoadedException
		try {
			tutor.addUser("JohnWatson", "biographer", "anotherdomain");
			
			System.err.println("Expected DomainNotLoadedException to be thrown!");
			fail();
		} catch (DomainNotLoadedException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing IllegalUserNameException
		try {
			tutor.addUser("John-Watson", "biographer", "german");
			
			System.err.println("Expected IllegalUserNameException to be thrown!");
			fail();
		} catch (IllegalUserNameException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing IllegalUserNameException again
		try {
			tutor.addUser("John>Watson", "biographer", "german");
			
			System.err.println("Expected IllegalUserNameException to be thrown!");
			fail();
		} catch (IllegalUserNameException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing StudentModelAlreadyExistsException
		try {
			tutor.addUser("JohnWatson", "biographer", "german");
			tutor.addUser("JohnWatson", "biographer", "german");
			
			System.err.println("Expected StudentModelAlreadyExistsException to be thrown!");
			fail();
		} catch (StudentModelAlreadyExistsException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public final void testRemoveUser() {
		
		// Setup
		try {
			tutor.addUser("JohnWatson", "biographer", "german");
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing successful removeUser
		try {
			assertTrue(tutor.getLoginStatus("SherlockHolmes", "german"));
			tutor.removeUser("SherlockHolmes", "german");
			assertFalse(tutor.getLoginStatus("SherlockHolmes", "german"));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		try {
			tutor.login("SherlockHolmes", "detective", "german");
			
			System.err.println("Expected InvalidLoginException to be thrown!");
			fail();
		} catch (InvalidLoginException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotFoundException
		try {
			tutor.removeUser("JohnWatson", "british");
			
			System.err.println("Expected DomainNotFoundException to be thrown!");
			fail();
		} catch (DomainNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing DomainNotLoadedException
		try {
			tutor.removeUser("JohnWatson", "anotherdomain");
			
			System.err.println("Expected DomainNotLoadedException to be thrown!");
			fail();
		} catch (DomainNotLoadedException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing UserNotFoundException
		try {
			tutor.removeUser("ConanDoyle", "german");
			
			System.err.println("Expected UserNotFoundException to be thrown!");
			fail();
		} catch (UserNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing UserNotLoggedInException
		try {
			assertFalse(tutor.getLoginStatus("JohnWatson", "german"));
			tutor.removeUser("JohnWatson", "german");
			
			System.err.println("Expected UserNotLoggedInException to be thrown!");
			fail();
		} catch (UserNotLoggedInException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}


	public final void testDeleteUser() {
		
		// Setup
		try {
			tutor.addUser("JohnWatson", "biographer", "german");
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing successful deleteUser
		try {
			assertTrue(tutor.getLoginStatus("SherlockHolmes", "german"));
			tutor.deleteUser("SherlockHolmes");
			assertFalse(tutor.getLoginStatus("SherlockHolmes", "german"));
		} catch (UserNotFoundException e) {
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		try {
			tutor.login("SherlockHolmes", "detective", "german");
			
			System.err.println("Expected UserNotFoundException to be thrown!");
			fail();
		} catch (UserNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		
		// Testing UserNotFoundException
		try {
			tutor.deleteUser("ConanDoyle");
			
			System.err.println("Expected UserNotFoundException to be thrown!");
			fail();
		} catch (UserNotFoundException e) {
			// Test Passed!
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public final void testAddAllowedDomain() {
		
		try {
			tutor.addUser("RichardHammond", "teethwhitener", "german");
			tutor.loadDomain("anotherdomain");
			tutor.addAllowedDomain("RichardHammond", "anotherdomain");
		} catch (Exception e) {
			System.out.println(e);
			fail(e.getMessage());
		}
		
		try {
			tutor.addUser("JamesMay", "teethwhitener", "german");
			tutor.addAllowedDomain("JamesMay", "kermit");
			fail();
		} catch (DomainNotLoadedException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.loadDomain("kermit");
			tutor.addAllowedDomain("RichardHammond", "kermit");
			fail();
		} catch (AddUserDeniedException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.addUser("JeremyClarkson", "teethwhitener", "german");
			tutor.addAllowedDomain("JeremyClarkson", "rubbish");
			fail();
		} catch (DomainNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		
		try {
			tutor.addAllowedDomain("RichardHammond", "anotherdomain");
			fail();
		} catch (StudentModelAlreadyExistsException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.addAllowedDomain("Noddy", "anotherdomain");
			fail();
		} catch (UserNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	public final void testChangePassword() {
		try {
			tutor.loadDomain("german");
			tutor.loadDomain("anotherdomain");
			tutor.addUser("RichardHammond", "teethwhitener", "german");
			
			tutor.login("RichardHammond", "teethwhitener", "german");
			tutor.changePassword("RichardHammond", "teethwhitener", "mini");
			tutor.logout("RichardHammond", "german");
			assertFalse(tutor.getLoginStatus("RichardHammond", "german"));
			tutor.login("RichardHammond", "mini", "german");
			tutor.addAllowedDomain("RichardHammond", "anotherdomain");
			tutor.login("RichardHammond", "mini", "anotherdomain");
			
			
		} catch (Exception e) {
			System.out.println(e);
			fail(e.getMessage());
		}
	
	
	
		try {
			tutor.changePassword("RichardHammond", "mini", "@##$$^");
		}
		catch (InvalidPasswordException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.changePassword("DavidAttenborough", "xxx", "yyy");
		}
		catch (UserNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	public final void testGetAllDomainNames() {
		try {
			tutor.getAllDomainNames();
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	

	public final void testGetSubDomains() {
		try {
			tutor.getSubdomains("german");
		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.getSubdomains("bundy");
		}
		catch (DomainNotFoundException e) {
			
		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.getSubdomains("kermit");
		}
		catch (DomainNotLoadedException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	public final void testLoadDomain() {
		try {
			tutor.loadDomain("kermit");
		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.loadDomain("bundy");
		}
		catch (DomainNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
	}
	
	public final void testSelectSubdomain() {
		try {
			tutor.addUser("JohnWatson", "biographer", "german");
			tutor.addUser("RichardHammond", "whiteteeth", "german");
			tutor.login("RichardHammond", "whiteteeth", "german");
			tutor.selectSubdomain("RichardHammond", "verbs", "german");
		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.selectSubdomain("RichardHammond", "verbs", "bundy");
		}
		catch (DomainNotFoundException e) {

		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		try {
			tutor.selectSubdomain("RichardHammond", "set1", "kermit");
		}
		catch (DomainNotLoadedException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.loadDomain("anotherdomain");
			tutor.addAllowedDomain("RichardHammond", "anotherdomain");
			tutor.login("RichardHammond", "whiteteeth", "anotherdomain");
			tutor.selectSubdomain("RichardHammond", "set1", "anotherdomain");
		}
		catch (ProblemsNotFoundException e) {

		} catch (Exception e) {
			System.out.println(e);
			fail(e.getMessage());
		}
		try {
			tutor.selectSubdomain("RichardHammond", "beerdrinking", "german");
		}
		catch (SubdomainNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.selectSubdomain("RobbieWilliams", "verbs", "german");
		}
		catch (UserNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.selectSubdomain("JohnWatson", "verbs", "german");
		}
		catch (UserNotLoggedInException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
	}
	
	
	public final void testGetNextProblem() {
		try {
			tutor.addUser("RichardHammond", "whiteteeth", "german");
			tutor.login("RichardHammond", "whiteteeth", "german");
			tutor.selectSubdomain("RichardHammond", "verbs", "german");
			Problem p = tutor.getNextProblem("RichardHammond", "german");
			
			/* when we select a subdomain we get given the first problem, so after calling
			getNextProblem we should be on the second one */
			assertTrue(p.getId() == 2);
			
			p = tutor.getNextProblem(3, "RichardHammond", "german");
			assertTrue(p.getId() == 3);
		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.getNextProblem("RichardHammond", "bundy");
		}
		catch (DomainNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.getNextProblem(12345, "RichardHammond", "german");
		}
		catch (InvalidProblemException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.getNextProblem(12345, "Ringo Starr", "german");
		}
		catch (UserNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			tutor.logout("RichardHammond", "german");
			tutor.getNextProblem(12345, "RichardHammond", "german");
		}
		catch (UserNotLoggedInException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
	}
	
	
	
	
	public final void testGetAllProblems() {
		try {
			
			tutor.addUser("RichardHammond", "whiteteeth", "german");
			tutor.login("RichardHammond", "whiteteeth", "german");
			tutor.selectSubdomain("RichardHammond", "verbs", "german");
			SortedMap<Integer, Problem> probs = tutor.getAllProblems("RichardHammond", "german");
			assertTrue(probs.size() == 55);
		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.getAllProblems("RichardHammond", "bundy");
		}
		catch (DomainNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.getAllProblems("RichardHammond", "kermit");
		}
		catch (DomainNotLoadedException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.getAllProblems("Tony Blair", "german");
		}
		catch (UserNotFoundException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
		
		try {
			tutor.logout("RichardHammond", "german");
			tutor.getAllProblems("RichardHammond", "german");
		}
		catch (UserNotLoggedInException e) {

		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
}
