package jitas.unittest;

import junit.framework.TestCase;
import jitas.core.*;
import jitas.exception.*;
import jitas.util.Logger;
import java.util.*;

/* 
 * @author	DeathMarch (c314g2)
 * @version	8 October 2007
 */
public class TutorTest extends TestCase {
	
	TutorInterface t;

	protected void setUp() throws Exception {
		super.setUp();
		t = new Tutor();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		/* the following ensures that at the start of each test user lpa24
		 * does not exist. */
		try {
			t.loadDomain("german");
			t.deleteUser("lpa24");
		} catch (UserNotFoundException e) {}
		t = null;
	}
	
	/* A test that tests the whole process */
	public void testEverything(){
		
		try {
			/* get all the domains */
			assertNotNull(t.getAllDomainNames());
			
			/* load the german domain */
			t.loadDomain("german");
			String d = "german";
			
			/* login then logout */
			t.login("david", "password", d);			
			assertTrue(t.getLoginStatus("david", d));
			t.logout("david", d);
			
			/* different cases for getLoginStatus() */
			t.login("david", "password", d);
			assertTrue(t.getLoginStatus("david", d));
			assertFalse(t.getLoginStatus("david", "kermit"));
			t.logout("david", d);
			
			try {
				/* create new user */
				t.addUser("newUser", "password", d);
				t.login("newUser", "password", d);
				assertTrue(t.getLoginStatus("newUser", d));
			} catch (StudentModelAlreadyExistsException e) {
				System.out.println("Exception is: " + e);
				t.login("newUser", "password", d);
			}
				
			t.selectSubdomain("newUser", "verbs", d);
			assertNotNull(t.getNextProblem("newUser",d));
			t.logout("newUser", d);
		} catch (InvalidLoginException e) {
			System.err.println(e.getMessage());
			fail();
			
		} catch (UserNotFoundException e) {
			System.err.println(e.getMessage());
			fail();
			
		} catch (UserNotLoggedInException e) {
			System.err.println(e.getMessage());
			fail();
			
		} catch (DomainNotFoundException e) {
			System.err.println(e.getMessage());
			fail();
		
		} catch (AddUserDeniedException e) {
			System.err.println(e.getMessage());
			fail();
		
		} catch (Exception e) {
			System.err.println(e.getMessage());
			fail();
		}
	}

	public void testTutor() {
		assertNotNull(t);
	}
	
	public void testAddUser2() throws Exception{
		t.loadDomain("german");
		t.addUser("abc", "bcd", "german");
		t.login("abc", "bcd", "german");
		t.removeUser("abc", "german");
	}
	
	public void testAddUser() {
		String d;
		try {
			t.loadDomain("german");
			d = "german";
			t.addUser("david", "password", d);
			fail();//the preceding line should have thrown an exception
		} catch (DomainNotFoundException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testAddUser: Domain not found");
		} catch (AddUserDeniedException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testAddUser: Add User Denied");
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
		
		try {
			t.loadDomain("german");
			d = "german";
			t.addUser("david^", "password", d);
			fail("TutorTest.testAddUser: Legal username");//the preceding line should have thrown an exception
		} catch (DomainNotFoundException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testAddUser: Domain not found");
		} catch (AddUserDeniedException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testAddUser: Add User Denied");
		} catch(IllegalUserNameException e){
			//Expect
		} catch (Exception e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testAddUser: Exception");
		}
	}
	
	public void testAddUserFails() {
		try {
			t.loadDomain("kermit");
			String d = "kermit";
			t.addUser("newUser", "password", d); //this should throw an AddUserDeniedException
			fail();
		} catch (AddUserDeniedException e) {
			return;
		} catch (Exception e) {
			System.err.println(e);
		}
		fail("TutorTest.testAddUser: Expected an AddUserDeniedException");
	}

	public void testLoadDomain() {
		try {
			t.loadDomain("german");
			String d = "german";
			assertNotNull(d);
		
			t.loadDomain("german");
			String d2 = "german";
			assertEquals(d,d2);
		} catch (DomainNotFoundException e) {
			System.err.println(e.getMessage());
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	
	
	public void testDomainNonexistent() {
		try {
			t.loadDomain("terminator");
			
		} catch (DomainNotFoundException e) {
			return;
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
		fail("No DomainNotFoundException thrown as expected.");
	}
	
	
	
	

	public void testGetDomains() {		
		Set<String> s;
		try {
			s = t.getAllDomainNames();
			Iterator<String> itr = s.iterator();
			/* at this stage we only have the german domain */
			assertTrue(itr.next().equals("kermit"));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	
	public void testSelectDomain() {
		
		try {
			t.loadDomain("german");
			t.login("newUser", "password", "german");
			
		} catch (DomainNotFoundException ex) {
			fail("testSelectDomain: Could not select the domain");
		} catch (InvalidLoginException e) {
			fail("Invalid login exception");
		} catch (UserNotFoundException e) {
			fail("user not found exception");
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	
	public void testGetLoginStatus() throws DomainNotFoundException, UserNotFoundException,
	InvalidLoginException, UserNotLoggedInException, Exception {
		
		t.loadDomain("german");
		String d = "german";
		t.login("david", "password", d);
		assertTrue(t.getLoginStatus("david", d));
		assertFalse(t.getLoginStatus("david", "kermit"));
		t.logout("david", d);
	}
	
	public void testRemoveUser(){	
		try{
			t.loadDomain("german");
			String d = "german";
			t.addUser("robert", "robert", d);
			t.login("robert", "robert", d);
			t.removeUser("robert", d);
			
			t.login("robert", "robert", d);
			fail("TutorTest.testRemoveUser: remove was unsucessful");
		} catch (DomainNotFoundException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testRemoveUser: Domain not found");
		} catch (AddUserDeniedException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testRemoveUser: Add User Denied");
		} catch(UserNotFoundException e){
			//Expect
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	
	public void testDeleteUser(){	
		try{
			t.loadDomain("german");
			String d = "german";
			
			t.addUser("robert", "robert", d);
			t.login("robert", "robert", d);
			t.removeUser("robert", d);
			
			t.deleteUser("robert");
			
			t.login("robert", "robert", d);
			fail("TutorTest.testRemoveUser: remove was unsucessful");
		} catch (DomainNotFoundException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testRemoveUser: Domain not found");
		} catch (AddUserDeniedException e) {
			System.err.println(e.getMessage());
			fail("TutorTest.testRemoveUser: Add User Denied");
		} catch(UserNotFoundException e){
			//Expect
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}	
	
	public void testGetNextProblemUser()  throws DomainNotFoundException, UserNotFoundException, 
	SubdomainNotFoundException, UserNotLoggedInException, InvalidLoginException, 
	ProblemsNotFoundException, InvalidProblemException, Exception {
		
		t.loadDomain("german");
		String d = "german";
		t.login("david", "password", d);
		t.selectSubdomain("david", "verbs", d);
		
		SortedMap<Integer, Problem> pl = t.getAllProblems("david", "german");
		Problem p = pl.get(36);
		Problem problem = t.getNextProblem(36,"david",d);	
		assertTrue(p.toString().equals(problem.toString()));
		
		/*Test the getProblem function that cannot be reached without login first*/
		t.logout("david", d);
		try{
			t.getNextProblem(37, "david", d);
			fail("TutorTest.testGetNextProblemUser: User not logged in");
		} catch(UserNotLoggedInException e){
			/*Expect it to fail*/
		}
	}

	
	public void testAddAllowedDomain() throws DomainNotFoundException, Exception {
		t.loadDomain("german");
		
		t.login("david", "password", "german");
		t.logout("david", "german");
					
		t.addUser("lpa24", "def", "german");
		t.login("lpa24", "def", "german");
		t.logout("lpa24", "german");
		
		
		try{
			t.login("lpa24", "def", "kermit");
			fail();
		} catch(DomainNotLoadedException e){
			
		}
		
		try {
			t.addAllowedDomain("lpa24", "some rubbish domain name");
			fail();
		} catch(DomainNotFoundException e){
			
		}
		
		try{	
			t.login("lpa24", "Def", "german");
			fail("TutorTest.testEncryption: Wrong password");
		} catch(InvalidLoginException e){
			/*Expect it to fail*/
		} finally {
			t.login("lpa24", "def", "german");
			t.removeUser("lpa24", "german");
		}
	}
	
	public void testGetNextProblemLexical() throws DomainNotFoundException, UserNotFoundException, 
	SubdomainNotFoundException, UserNotLoggedInException, InvalidLoginException, 
	ProblemsNotFoundException, InvalidProblemException, Exception {
		
		t.loadDomain("german");
		String d = "german";
		t.login("david", "password", d);
		t.selectSubdomain("david", "verbs", d);
		
		SortedMap<Integer, Problem> pl = t.getAllProblems("david", "german");
		
		//the first 2 problems have been attempted, so the third one should be returned by getNextProblem
		Problem p = pl.get(3);

		Problem problem = t.getNextProblem("david",d);
		assertTrue(p.toString().equals(problem.toString())); 
		
		/*Test the getProblem function that cannot be reached without login first*/
		t.logout("david", d);
		try{
			t.getNextProblem("david",d);
			fail("TutorTest.testGetNextProblemLexical: User not logged in");
		} catch(UserNotLoggedInException e){
			/*Expect it to fail*/
		}
	}
	
	public void testGetAllProblems() throws DomainNotFoundException, UserNotFoundException, 
	SubdomainNotFoundException, UserNotLoggedInException, InvalidLoginException, 
	ProblemsNotFoundException, InvalidProblemException, Exception {
		
		t.loadDomain("german");
		String d = "german";
		t.login("david", "password", d);
		t.selectSubdomain("david", "verbs", d);
		
		
		SortedMap<Integer, Problem> pl = t.getAllProblems("david", "german");

		SortedMap<Integer, Problem> pl2 = t.getAllProblems("david", d);
		for(int i = 0; i < pl.size(); i++){
			assertTrue(pl2.get(i+1).toString().equals(pl.get(i+1).toString()));
		}
		
		/*Test the getProblem function that cannot be reached without login first*/
		t.logout("david", d);
		try{
			t.getAllProblems("david",d);
			fail("TutorTest.testGetNextProblemLexical: User not logged in");
		} catch(UserNotLoggedInException e){
			/*Expect it to fail*/
		}
	}

	public void testEncryption() throws DomainNotFoundException, Exception {
		t.loadDomain("german");
		
		t.login("david", "password", "german");
		t.logout("david", "german");
			
		t.addUser("lpa24", "def", "german");
		t.login("lpa24", "def", "german");
		t.logout("lpa24", "german");
		
		try{	
			t.login("lpa24", "Def", "german");
			fail("TutorTest.testEncryption: Wrong password");
		} catch(InvalidLoginException e){
			/*Expect it to fail*/
		} finally {
			t.login("lpa24", "def", "german");
			t.removeUser("lpa24", "german");
		}
	}
	

	public void testChangePassword() throws DomainNotFoundException, Exception{
		t.loadDomain("german");
		t.addUser("lpa24", "def", "german");
		t.login("lpa24", "def", "german");
		t.changePassword("lpa24","def", "password");
		t.logout("lpa24", "german");

		try {

			t.login("lpa24", "def", "german");

			fail("TutorTest.testChangePassword: Password has been changed");
		} catch(InvalidLoginException e){
			//Expect it to fail
		}
		try {

			t.login("lpa24", "password","german");

		}catch(InvalidLoginException e){
			System.err.println(e.getMessage());
			fail("TutorTest.testChangePassword: Invalid login");
		}
		t.changePassword("lpa24","password", "def");
		t.removeUser("lpa24", "german");
	}

	public void testSubmitSolution() {
		
		try {
			t.loadDomain("german");
			t.login("david", "password", "german");
			t.selectSubdomain("david", "verbs", "german");
			t.getNextProblem(1, "david", "german");
			
			Feedback fb = t.submitSolution(new Solution("((ANSWER \"graue\")(ARTICLE \"C\")(CASE \"N\")(GENDER \"Y\"))"), "david", "german");
			assertTrue(fb.getMessages().size() == 3);
			
			Feedback fb1 = t.submitSolution(new Solution("((ANSWER \"g r a u e\")(ARTICLE \"D\")(CASE \"N\")(GENDER \"N\"))"), "david", "german");
			assertTrue(fb1.getMessages().size() == 0);
			
			t.getNextProblem(2, "david", "german");
			Feedback fb2 = t.submitSolution(new Solution("((ARTICLE \"D\")(ANSWER \"graue\"))"), "david", "german");
			assertTrue(fb2.getMessages().size() == 5);
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println(e);
			fail();
		}	
	}
	
	public void testLogger() {
		Logger.addEntry("Something has gone terribly, terribly wrong");
		Logger.clearLog();
	}
	
	public void testGetAllLongDomainNames() {
		try {
			t.loadDomain("german");
			t.loadDomain("kermit");
			Map<String, String> m = t.getAllLongDomainNames();
			assert(m.get("german").equals("The German Tutor"));
			assert(m.get("kermit").equals("The Database Design Tutor"));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	public void testGetProblemStatus() {
		try {
			t.loadDomain("german");
			t.login("david", "password", "german");
			t.selectSubdomain("david", "verbs", "german");
			t.getNextProblem(1, "david", "german");
			t.submitSolution(new Solution("((CASE \"D A\")(ANSWER \"graue\"))"), "david", "german");
			assertTrue(t.getProblemStatus("david", "german", 1).equals("attempted"));
			t.getNextProblem(3, "david", "german");
			assertTrue(t.getProblemStatus("david", "german", 3).equals("not attempted"));
		} catch (Exception e) {
			fail(e.getMessage());
		}
		
	}
	
	public void testRandomStuff() {
		try {
			t.loadDomain("german");
			t.addUser("lpa24", "password", "german");
			t.login("lpa24", "password", "german");
			t.selectSubdomain("lpa24", "verbs", "german");
			t.getNextProblem(1, "lpa24", "german");
			t.getNextProblem("lpa24", "german");
			t.getNextProblem("lpa24", "german");
			t.getNextProblem(2, "lpa24", "german");
			t.submitSolution(new Solution("((CASE \"N\")(GENDER \"M\")(ARTICLE \"I\")(ANSWER \"g r o s s e r\"))"), "lpa24", "german");
			t.getNextProblem(1, "lpa24", "german");
			t.getNextProblem("lpa24", "german");
			t.removeUser("lpa24", "german");
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			t.login("lpa24", "password", "german");
			fail();
		} catch (InvalidLoginException e) {
		} catch (Exception e) {
			fail();
		}
	}
	
	public void testCustomConstraints() {
		try {
			t.loadDomain("german");
			t.login("david", "password", "german");
			t.selectSubdomain("david", "verbs", "german");
			t.getNextProblem(1, "david", "german");
			t.submitSolution(new Solution("((ARTICLE \"D\")(ANSWER \"graue\"))"), "david", "german");
			t.addUser("lpa24", "password", "german");
			t.login("lpa24", "password", "german");
			t.selectSubdomain("lpa24", "verbs", "german");
			t.getNextProblem(1, "lpa24", "german");
			t.submitSolution(new Solution("((ARTICLE \"D\")(ANSWER \"graue\"))"), "lpa24", "german");
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
	}
	
	public void testStudentModelParser(){
		try{
			t.loadDomain("german");
			t.login("david", "password", "german");
			t.selectSubdomain("david", "verbs", "german");
			t.submitSolution(new Solution("((CASE \"Y\")(GENDER \"N\"))"), "david", "german");
		} catch(Exception e){
			e.printStackTrace();
			fail();
		}
	}
	
	public void testSystemProblemSelection(){
		try{
			t.loadDomain("german");
			t.login("david", "password", "german");
			t.selectSubdomain("david", "verbs", "german");
			t.setCustomProblemSelectionAlgorithm("german", "jitas.core.DefaultSelectionAlgorithm");
			t.setCustomFeedback("german", "jitas.core.SimpleFeedback");
			Problem pro = t.getProblem("david", "german");
			System.out.println(pro.toString());
		} catch(Exception e){
			e.printStackTrace();
			fail();
		}
	}
}
